from __future__ import absolute_import, division, print_function

__all__ = [
    '__title__', '__summary__', '__uri__', '__version__', '__author__',
    '__email__', '__license__', '__copyright__',
]

__title__ = 'trailblazer-aws'
__summary__ = ('Blazing Cloudtrail since 2018')
__uri__ = 'https://github.com/willbengtson/trailblazer-aws'

__version__ = '1.0.1'

__author__ = 'Will Bengtson'
__email__ = 'william.bengtson@gmail.com'

__license__ = 'Apache License, Version 2.0'
__copyright__ = 'Copyright 2018 {0}'.format(__author__)
\